# $Id: deferrable.rb 320 2007-05-22 22:12:59Z blackhedd $
#
# Author:: Francis Cianfrocca (gmail: blackhedd)
# Homepage::  http://rubyeventmachine.com
# Date:: 16 Jul 2006
# 
# See EventMachine and EventMachine::Connection for documentation and
# usage examples.
#
#----------------------------------------------------------------------------
#
# Copyright (C) 2006-07 by Francis Cianfrocca. All Rights Reserved.
# Gmail: blackhedd
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of either: 1) the GNU General Public License
# as published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version; or 2) Ruby's License.
# 
# See the file COPYING for complete licensing information.
#
#---------------------------------------------------------------------------
#
# 

require 'forwardable'

module EventMachine

module Deferrable
  def callback &block
    return unless block
    if @deferred_status == :succeeded
      block.call(*@deferred_args)
    else
      @callbacks ||= []
      @callbacks << block
    end
  end

  def errback &block
    return unless block
    if @deferred_status == :failed
      block.call(*@deferred_args)
    else
      @errbacks ||= []
      @errbacks << block
    end
  end

  # Note that if you call this method without arguments,
  # no arguments will be passed to the callback/errback.
  # If the user has coded these with arguments, then the
  # user code will throw an argument exception.
  # Implementors of deferrable classes <b>must</b>
  # document the arguments they will supply to user callbacks.
  #
  # OBSERVE SOMETHING VERY SPECIAL here: you may call this method even
  # on the INSIDE of a callback. This is very useful when a previously-registered
  # callback wants to change the parameters that will be passed to subsequently-registered
  # ones.
  # --
  # We're shifting callbacks off and discarding them as we execute them.
  # This is valid because by definition callbacks are executed no more than
  # once. It also has the magic effect of permitting recursive calls, which
  # means that a callback can call #set_deferred_status and change the parameters
  # that will be sent to subsequent callbacks down the chain.
  #
  def set_deferred_status arg, *args
    @deferred_status = arg
    @deferred_args = args
    case @deferred_status
    when :succeeded
      if @callbacks
        while cb = @callbacks.shift
          cb.call(*@deferred_args)
        end
      end
    when :failed
      if @errbacks
        while eb = @errbacks.shift
          eb.call(*@deferred_args)
        end
      end
    end
  end

  # Equivalent to set_deferred_status(:succeeded, ...)
  #
  def set_deferred_success *args
	  set_deferred_status :succeeded, *args
  end

  # Equivalent to set_deferred_status(:failed, ...)
  #
  def set_deferred_failure *args
	  set_deferred_status :failed, *args
  end

end
end

