# $Id: test_futures.rb 323 2007-05-22 22:22:43Z blackhedd $
#
# Author:: Francis Cianfrocca (gmail: blackhedd)
# Homepage::  http://rubyeventmachine.com
# Date:: 8 April 2006
# 
# See EventMachine and EventMachine::Connection for documentation and
# usage examples.
#
#----------------------------------------------------------------------------
#
# Copyright (C) 2006-07 by Francis Cianfrocca. All Rights Reserved.
# Gmail: blackhedd
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of either: 1) the GNU General Public License
# as published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version; or 2) Ruby's License.
# 
# See the file COPYING for complete licensing information.
#
#---------------------------------------------------------------------------
#
#
#

$:.unshift "../lib"
require 'eventmachine'



class TestFutures < Test::Unit::TestCase

  def setup
  end

  def teardown
  end

  def test_future
      assert_equal(100, EventMachine::Deferrable.future(100) )

      p1 = proc { 100 + 1 }
      assert_equal(101, EventMachine::Deferrable.future(p1) )
  end



  class MyFuture
      include EventMachine::Deferrable
      def initialize *args
	  super
	  set_deferred_status :succeeded, 40
      end
  end

  class MyErrorFuture
      include EventMachine::Deferrable
      def initialize *args
	  super
	  set_deferred_status :failed, 41
      end
  end


  def test_future_1
      # Call future with one additional argument and it will be treated as a callback.
      def my_future
	  MyFuture.new
      end

      value = nil
      EventMachine::Deferrable.future my_future, proc {|v| value=v}
      assert_equal( 40, value )
  end


  def test_future_2
      # Call future with two additional arguments and they will be treated as a callback
      # and an errback.
      value = nil
      EventMachine::Deferrable.future MyErrorFuture.new, nil, proc {|v| value=v}
      assert_equal( 41, value )
  end


  def test_future_3
      # Call future with no additional arguments but with a block, and the block will be
      # treated as a callback.
      value = nil
      EventMachine::Deferrable.future MyFuture.new do |v|
	  value=v
      end
      assert_equal( 40, value )
  end


  class RecursiveCallback
      include EventMachine::Deferrable
  end

  # A Deferrable callback can call #set_deferred_status to change the values
  # passed to subsequent callbacks.
  #
  def test_recursive_callbacks
      n = 0 # counter assures that all the tests actually run.
      rc = RecursiveCallback.new
      rc.callback {|a|
	  assert_equal(100, a)
	  n += 1
	  rc.set_deferred_status :succeeded, 101, 101
      }
      rc.callback {|a,b|
	  assert_equal(101, a)
	  assert_equal(101, b)
	  n += 1
	  rc.set_deferred_status :succeeded, 102, 102, 102
      }
      rc.callback {|a,b,c|
	  assert_equal(102, a)
	  assert_equal(102, b)
	  assert_equal(102, c)
	  n += 1
      }
      rc.set_deferred_status :succeeded, 100
      assert_equal(3, n)
  end

  def test_syntactic_sugar
	  rc = RecursiveCallback.new
	  rc.set_deferred_success 100
	  rc.set_deferred_failure 200
  end

end

